import { ValidateBy, ValidationOptions, buildMessage } from 'class-validator';
import { AddressDTO } from '../create.address.dto';

export function hasProperPositionIndexes(value: AddressDTO[]) {
  if (value.every((address) => address.position === undefined)) return true;
  else if (value.every((address) => typeof address.position === 'number')) {
    value.sort((a, b) => a.position - b.position);
    for (let i = 1; i < value.length; i++) {
      if (value[i - 1].position >= value[i].position) return false;
    }
    return true;
  }
  return false;
}

export function HasProperPositionIndexes(
  validationOptions?: ValidationOptions,
): PropertyDecorator {
  return ValidateBy(
    {
      name: '',
      constraints: [],
      validator: {
        validate: (value: AddressDTO[]): boolean =>
          hasProperPositionIndexes(value),
        defaultMessage: buildMessage(
          () =>
            `indexes position incorrect, please provide a complete list of indexes or ordened list of adresses from start to end of journey`,
          validationOptions,
        ),
      },
    },
    validationOptions,
  );
}
