import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Inject } from '@nestjs/common';
import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { AdsRepository } from '../../adapters/secondaries/ads.repository';
import { CreateAdCommand } from '../../commands/create-ad.command';
import { CreateAdRequest } from '../dtos/create-ad.request';
import { Ad } from '../entities/ad';
import { IProvideParams } from '../interfaces/param-provider.interface';
import { DefaultParams } from '../types/default-params.type';
import { Address } from '../entities/address';

@CommandHandler(CreateAdCommand)
export class CreateAdUseCase {
  private readonly defaultParams: DefaultParams;
  private ad: Ad;
  constructor(
    private readonly _repository: AdsRepository,
    private readonly _messager: Messager,
    @InjectMapper() private readonly _mapper: Mapper,
    @Inject('ParamsProvider')
    private readonly defaultParamsProvider: IProvideParams,
  ) {
    this.defaultParams = defaultParamsProvider.getParams();
  }

  async execute(command: CreateAdCommand): Promise<Ad> {
    console.log('before mapping ');
    const entity: Ad = this._mapper.map(
      command.createAdRequest,
      CreateAdRequest,
      Ad,
    );
    typeof entity.monMargin === 'number'
      ? entity.monMargin
      : (entity.monMargin = this.defaultParams.MON_MARGIN);
    typeof entity.tueMargin === 'number'
      ? entity.tueMargin
      : (entity.tueMargin = this.defaultParams.TUE_MARGIN);
    typeof entity.wedMargin === 'number'
      ? entity.wedMargin
      : (entity.wedMargin = this.defaultParams.WED_MARGIN);
    typeof entity.thuMargin === 'number'
      ? entity.thuMargin
      : (entity.thuMargin = this.defaultParams.THU_MARGIN);
    typeof entity.friMargin === 'number'
      ? entity.friMargin
      : (entity.friMargin = this.defaultParams.FRI_MARGIN);
    typeof entity.satMargin === 'number'
      ? entity.satMargin
      : (entity.satMargin = this.defaultParams.SAT_MARGIN);
    typeof entity.sunMargin === 'number'
      ? entity.sunMargin
      : (entity.sunMargin = this.defaultParams.SUN_MARGIN);
    typeof entity.strict === 'boolean'
      ? entity.strict
      : (entity.strict = this.defaultParams.STRICT);

    if (typeof entity.addresses[0].position === 'undefined') {
      for (let i = 0; i < entity.addresses.length; i++) {
        entity.addresses[i].position = i;
      }
    }
    try {
      console.log('before ***********************************');
      console.log(entity);
      console.log('******************************************');
      this.ad = await this._repository.create(entity);
      // this._messager.publish('ad.create', JSON.stringify(ad));
      // this._messager.publish('logging.ad.create.info', JSON.stringify(ad));
      return this.ad;
    } catch (error) {
      let key = 'logging.ad.create.crit';
      if (error.message.includes('Already exists')) {
        key = 'logging.ad.create.warning';
      }
      this._messager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
