import { createMap, forMember, mapFrom, Mapper } from '@automapper/core';
import { AutomapperProfile, InjectMapper } from '@automapper/nestjs';
import { Injectable } from '@nestjs/common';
import { Ad } from '../domain/entities/ad';
import { AdPresenter } from '../adapters/primaries/ad.presenter';
import { CreateAdRequest } from '../domain/dtos/create-ad.request';
import { Address } from '../domain/entities/address';
import { AddressDTO } from '../domain/dtos/create.address.dto';

@Injectable()
export class AdProfile extends AutomapperProfile {
  constructor(@InjectMapper() mapper: Mapper) {
    super(mapper);
  }

  override get profile() {
    return (mapper) => {
      createMap(mapper, Address, AddressDTO);
      createMap(mapper, Ad, AdPresenter);
      createMap(
        mapper,
        CreateAdRequest,
        Ad,
        forMember(
          (destination) => destination.monMargin,
          mapFrom((source) => source.marginDurations.mon),
        ),
        forMember(
          (destination) => destination.tueMargin,
          mapFrom((source) => source.marginDurations.tue),
        ),
        forMember(
          (destination) => destination.wedMargin,
          mapFrom((source) => source.marginDurations.wed),
        ),
        forMember(
          (destination) => destination.thuMargin,
          mapFrom((source) => source.marginDurations.thu),
        ),
        forMember(
          (destination) => destination.friMargin,
          mapFrom((source) => source.marginDurations.fri),
        ),
        forMember(
          (destination) => destination.satMargin,
          mapFrom((source) => source.marginDurations.sat),
        ),
        forMember(
          (destination) => destination.sunMargin,
          mapFrom((source) => source.marginDurations.sun),
        ),
        forMember(
          (destination) => destination.monTime,
          mapFrom((source) => source.schedule.mon),
        ),
        forMember(
          (destination) => destination.tueTime,
          mapFrom((source) => source.schedule.tue),
        ),
        forMember(
          (destination) => destination.wedTime,
          mapFrom((source) => source.schedule.wed),
        ),
        forMember(
          (destination) => destination.thuTime,
          mapFrom((source) => source.schedule.thu),
        ),
        forMember(
          (destination) => destination.friTime,
          mapFrom((source) => source.schedule.fri),
        ),
        forMember(
          (destination) => destination.satTime,
          mapFrom((source) => source.schedule.sat),
        ),
        forMember(
          (destination) => destination.sunTime,
          mapFrom((source) => source.schedule.sun),
        ),
        forMember(
          (destination) => destination.addresses,
          mapFrom((source) => source.addresses),
        ),
      );
    };
  }
}
